package simulation;

import metadata.BuildingMetadata;

import java.util.Optional;

public abstract class Building {
    protected final int id;
    protected final String type;
    protected final int x;
    protected final int y;
    protected BuildingState state = BuildingState.EMPTY;

    protected Building(int id, String type, int x, int y) {
        this.id = id;
        this.type = type;
        this.x = x;
        this.y = y;
    }

    public abstract void processInput(Component input);

    /**
     * Continue la production.
     * @return Le composant produit. Si le bâtiment n'a aucun produit fini ce tour ci, l'Optional retourné sera vide.
     */
    public abstract Optional<Component> produce();
    public abstract BuildingMetadata getMetadata();

    public int getId() {
        return id;
    }

    public String getType() {
        return type;
    }

    public int getX() {
        return x;
    }

    public int getY() {
        return y;
    }

    public BuildingState getState() {
        return state;
    }
}
