package view;

import configuration.SimulationConfiguration;
import configuration.SimulationConfigurationSingleton;
import configuration.SimulationData;
import simulation.Building;
import simulation.Component;
import simulation.Route;
import view.components.BuildingIcon;
import view.components.ComponentIcon;
import view.components.RouteLine;

import javax.swing.*;
import java.awt.*;
import java.io.Serial;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class MainPanel extends JLayeredPane {
    @Serial
    private static final long serialVersionUID = 1L;

    private final Map<Integer, Building> buildingsById = new HashMap<>();
    private final Collection<ComponentIcon> componentIcons = new ArrayList<>();

    private SimulationConfiguration configuration;
    private SimulationData simulationData;

    public void reloadSimulation() {
        configuration = SimulationConfigurationSingleton.getInstance().getConfiguration();
        simulationData = configuration.getSimulationData();

        buildingsById.clear();
        componentIcons.clear();
        removeAll();
        repaint();

        int routesOffset = 0;   // Cet offset permet aux lignes de routes d'être au centre des icônes des bâtiments.
        for (Building building : simulationData.getBuildings()) {
            BuildingIcon icon = this.initializeBuilding(building);
            routesOffset = icon.getWidth() / 2;
        }

        for (Route route : simulationData.getRoutes()) {
            initializeRoute(route, routesOffset);
        }
    }

    private BuildingIcon initializeBuilding(Building building) {
        buildingsById.put(building.getId(), building);

        BuildingIcon icon = new BuildingIcon(building);
        add(icon);
        setLayer(icon, 2);

        return icon;
    }

    private void initializeRoute(Route route, int offset) {
        Building from = getBuildingById(route.getFrom());
        Building to = getBuildingById(route.getTo());

        RouteLine line = new RouteLine(from, to, offset);

        add(line);
        setLayer(line, 0);
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (configuration == null) return;

        updateBuildings();
        updateComponentIcons();
    }

    private void updateComponentIcons() {
        Collection<ComponentIcon> removedIcons = new ArrayList<>();

        componentIcons.forEach(i -> {
            i.updatePosition();

            boolean destinationReached = i.isAtDestination();
            if (destinationReached) {
                removedIcons.add(i);
            }
        });

        removedIcons.forEach(i -> {
            componentIcons.remove(i);
            remove(i);

            pushComponentToBuilding(i.getComponent(), i.getDestinationId());
        });
    }

    private void pushComponentToBuilding(Component c, int buildingId) {
        Building outputBuilding = getBuildingById(buildingId);
        outputBuilding.processInput(c);
    }

    private void updateBuildings() {
        simulationData.getBuildings().forEach(b -> {
            b.produce().ifPresent(c -> addComponentRoute(c, b));
        });
    }

    private void addComponentRoute(Component component, Building fromBuilding) {
        Route route = simulationData.getRoutes().stream()
                .filter(r -> r.getFrom() == fromBuilding.getId())
                .findFirst().get();

        Building toBuilding = buildingsById.get(route.getTo());
        Point fromPosition = new Point(fromBuilding.getX(), fromBuilding.getY());
        Point toPosition = new Point(toBuilding.getX(), toBuilding.getY());

        ComponentIcon icon = new ComponentIcon(component, route.getTo(), fromPosition, toPosition);
        componentIcons.add(icon);

        add(icon);
        setLayer(icon, 1);
    }

    private Building getBuildingById(int id) {
        return buildingsById.get(id);
    }
}
