package view;

import configuration.SimulationConfigurationSingleton;
import simulation.RandomSellStrategy;
import simulation.SellStrategy;
import simulation.TimedSellStrategy;
import simulation.Warehouse;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.Serial;
import java.util.Enumeration;

public class StrategyPanel extends JPanel {
    @Serial
    private static final long serialVersionUID = 1L;

    public StrategyPanel() {
        JRadioButton strategy1 = new JRadioButton("Interval");
        JRadioButton strategy2 = new JRadioButton("Aléatoire");

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(strategy1);
        buttonGroup.add(strategy2);

        JButton confirmButton = new JButton("Confirmer");
        confirmButton.addActionListener((ActionEvent e) -> {
            String selectedStrategyName = getSelectedButtonText(buttonGroup);
            onStrategyChange(selectedStrategyName);

            // Fermer la fenêtre du component
            SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
        });

        JButton cancelButton = new JButton("Annuler");
        cancelButton.addActionListener((ActionEvent e) -> {
            // Fermer la fenêtre du component
            SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
        });

        add(strategy1);
        add(strategy2);
        add(confirmButton);
        add(cancelButton);
    }

    /**
     * Retourne le bouton sélectionné dans un groupe de boutons.
     *
     * @param buttonGroup
     * @return
     */
    public String getSelectedButtonText(ButtonGroup buttonGroup) {
        for (Enumeration<AbstractButton> buttons = buttonGroup.getElements(); buttons.hasMoreElements(); ) {
            AbstractButton button = buttons.nextElement();
            if (button.isSelected()) {
                return button.getText();
            }
        }

        return null;
    }

    private void onStrategyChange(String strategyName) {
        SellStrategy strategy = strategyName.equalsIgnoreCase("Interval") ?
                new TimedSellStrategy() :
                new RandomSellStrategy();

        Warehouse warehouse = SimulationConfigurationSingleton.getInstance().getConfiguration().getSimulationData().getWarehouse();
        warehouse.setSellStrategy(strategy);
    }
}
