package view;

import configuration.*;

import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileSystemView;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.Serial;

public class WindowMenu extends JMenuBar {
    @Serial
    private static final long serialVersionUID = 1L;
    private static final String MENU_FILE_TITLE = "Fichier";
    private static final String MENU_FILE_LOAD = "Charger";
    private static final String MENU_FILE_EXIT = "Quitter";
    private static final String MENU_SIMULATION_TITLE = "Simulation";
    private static final String MENU_SIMULATION_CHOOSE = "Choisir";
    private static final String MENU_HELP_TITLE = "Aide";
    private static final String MENU_HELP_ABOUT = "À propos de...";

    private final ConfigurationParser configParser = new XmlConfigurationParser();
    private final MainPanel mainPanel;

    public WindowMenu(MainPanel mainPanel) {
        this.mainPanel = mainPanel;

        addFileMenu();
        addSimulationMenu();
        addHelpMenu();
    }

    /**
     * Créer le menu de fichier.
     */
    private void addFileMenu() {
        JMenu fileMenu = new JMenu(MENU_FILE_TITLE);
        JMenuItem loadMenu = new JMenuItem(MENU_FILE_LOAD);
        JMenuItem exitMenu = new JMenuItem(MENU_FILE_EXIT);

        loadMenu.addActionListener((ActionEvent e) -> {
            JFileChooser fileChooser = new JFileChooser(FileSystemView.getFileSystemView().getHomeDirectory());
            fileChooser.setDialogTitle("Sélectionnez un fichier de configuration");
            fileChooser.setAcceptAllFileFilterUsed(false);
            // Créer un filtre
            FileNameExtensionFilter filter = new FileNameExtensionFilter(".xml", "xml");
            fileChooser.addChoosableFileFilter(filter);

            int returnValue = fileChooser.showOpenDialog(null);
            if (returnValue == JFileChooser.APPROVE_OPTION) {
                File selectedFile = fileChooser.getSelectedFile();

                try {
                    SimulationConfiguration configuration = configParser.parseConfiguration(selectedFile);
                    SimulationConfigurationSingleton.getInstance().setConfiguration(configuration);

                    mainPanel.reloadSimulation();
                } catch (ConfigurationParsingException ex) {
                    throw new RuntimeException("Failed to parse config file", ex);
                }
            }
        });

        exitMenu.addActionListener((ActionEvent e) -> {
            System.exit(0);
        });

        fileMenu.add(loadMenu);
        fileMenu.add(exitMenu);

        add(fileMenu);
    }

    /**
     * Créer le menu de simulation.
     */
    private void addSimulationMenu() {
        JMenu simulationMenu = new JMenu(MENU_SIMULATION_TITLE);
        JMenuItem chooseMenu = new JMenuItem(MENU_SIMULATION_CHOOSE);

        chooseMenu.addActionListener((ActionEvent e) -> {
            // Ouvrir la fenêtre de sélection
            new StrategyWindow();
        });

        simulationMenu.add(chooseMenu);

        add(simulationMenu);
    }

    /**
     * Créer le menu aide.
     */
    private void addHelpMenu() {
        JMenu helpMenu = new JMenu(MENU_HELP_TITLE);
        JMenuItem aboutMenu = new JMenuItem(MENU_HELP_ABOUT);

        aboutMenu.addActionListener((ActionEvent e) -> {
            JOptionPane.showMessageDialog(null,
                    "<html><p>Application de simulation une chaine de production d'avions.</p><br>" +
                            "<p>&copy; &nbsp; 2017 &nbsp; Ghizlane El Boussaidi</p>" +
                            "<p>&copy; &nbsp; 2017 &nbsp; Dany Boisvert</p>" +
                            "<p>&copy; &nbsp; 2017 &nbsp; Vincent Mattard</p><br>" +
                            "<p>&Eacute;cole de technologie sup&eacute;rieure</p></html>");
        });

        helpMenu.add(aboutMenu);

        add(helpMenu);
    }
}
