package view.components;

import metadata.BuildingMetadata;
import simulation.Building;
import simulation.BuildingState;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class BuildingIcon extends JComponent {
    private static final Map<String, Map<BuildingState, Image>> loadedImages = new HashMap<>();

    private final Building building;
    private final Map<BuildingState, Image> images;

    public BuildingIcon(Building building) {
        this.building = building;
        this.images = getImages(building.getMetadata());

        setBounds();
    }

    @Override
    protected void paintComponent(Graphics g) {
        Image image = getCurrentStateImage();

        g.drawImage(image, 0, 0, null);
    }

    public Image getCurrentStateImage() {
        return images.get(building.getState());
    }

    private void setBounds() {
        // Présume que toutes les icônes ont les mêmes dimensions
        Image image = images.get(BuildingState.EMPTY);

        int x = building.getX();
        int y = building.getY();
        int width = image.getWidth(null);
        int height = image.getHeight(null);

        setBounds(x, y, width, height);
    }

    private static Map<BuildingState, Image> getImages(BuildingMetadata metadata) {
        if (loadedImages.containsKey(metadata.getType())) {
            return loadedImages.get(metadata.getType());
        }

        return loadImages(metadata);
    }

    private static Map<BuildingState, Image> loadImages(BuildingMetadata metadata) {
        Map<BuildingState, Image> images = new HashMap<>();

        for (BuildingState state : BuildingState.values()) {
            String path = metadata.getIconsPaths().get(state);
            Image image = loadImage(path);

            images.put(state, image);
        }

        loadedImages.put(metadata.getType(), images);
        return images;
    }

    private static Image loadImage(String path) {
        try {
            File file = new File(path);
            return ImageIO.read(file);
        } catch (IOException ex) {
            throw new RuntimeException("Could not load building icon", ex);
        }
    }
}
