package view.components;

import simulation.ComponentType;
import simulation.Component;
import view.MainWindow;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class ComponentIcon extends JComponent {
    private static final Map<ComponentType, Image> loadedImages = new HashMap<>();
    private static final int SPEED = 1;

    private final Component component;
    private final int destinationId;
    private final Point position;
    private final Point destination;
    private final Point direction;
    private final Image image;

    public ComponentIcon(Component component, int destinationId, Point source, Point destination) {
        this.component = component;
        this.destinationId = destinationId;
        this.position = source;
        this.destination = destination;

        this.direction = getDirection();
        this.image = getImage(component.getType());

        setBounds(0, 0, MainWindow.DIMENSION.width, MainWindow.DIMENSION.height);
    }

    @Override
    public void paintComponent(Graphics g) {
        g.translate(position.x, position.y);
        g.drawImage(image, 0, 0, null);
    }

    public void updatePosition() {
        position.translate(direction.x, direction.y);
    }

    public boolean isAtDestination() {
        return position.equals(destination);
    }

    public Component getComponent() {
        return component;
    }

    public int getDestinationId() {
        return destinationId;
    }

    private Point getDirection() {
        return new Point(
                normalizedDirection(position.x, destination.x),
                normalizedDirection(position.y, destination.y)
        );
    }

    private static int normalizedDirection(int from, int to) {
        // compare retourne 1, 0 ou -1 en comparant to à from.
        // Il suffit de multiplier ce résultat par la vitesse désirée pour trouver la direction normalisée.
        return Integer.compare(to, from) * SPEED;
    }

    private static Image getImage(ComponentType type) {
        if (loadedImages.containsKey(type)) {
            return loadedImages.get(type);
        }

        return loadImage(type);
    }

    private static Image loadImage(ComponentType type) {
        try {
            String path = String.format("src/ressources/%s.png", type.getTypeName());
            File file = new File(path);
            Image image = ImageIO.read(file);

            // Garde une référence à l'image chargée pour pouvoir la réutiliser la prochaine fois
            loadedImages.put(type, image);

            return image;
        } catch (IOException ex) {
            throw new RuntimeException("Could not load component icon", ex);
        }
    }
}
