package view.components;

import simulation.Building;

import javax.swing.*;
import java.awt.*;

public class RouteLine extends JComponent {
    private final Point from;
    private final Point to;

    public RouteLine(Building fromBuilding, Building toBuilding, int offset) {
        from = new Point(fromBuilding.getX(), fromBuilding.getY());
        to = new Point(toBuilding.getX(), toBuilding.getY());

        setBounds(from, to, offset);

        int offsetFromOriginX = Math.min(from.x, to.x);
        int offsetFromOriginY = Math.min(from.y, to.y);

        // La position de la ligne dépend de x et y des bounds du JComponent.
        // Ainsi, la position [0 0] est en fait [x y] des bounds.
        // Il faut donc retirer ces positions pour ne pas dessiner en dehors du rectangle de dessin.
        from.translate(-offsetFromOriginX, -offsetFromOriginY);
        to.translate(-offsetFromOriginX, -offsetFromOriginY);
    }

    @Override
    protected void paintComponent(Graphics g) {
        g.drawLine(from.x, from.y, to.x, to.y);
    }

    private void setBounds(Point from, Point to, int offset) {
        int x = Math.min(from.x, to.x);
        int y = Math.min(from.y, to.y);
        int width = Math.abs(to.x - from.x);
        int height = Math.abs(to.y - from.y);

        // Si une des dimensions est zéro, la ligne n'apparaîtra pas.
        if (width == 0) {
            width = 1;
        }

        if (height == 0) {
            height = 1;
        }

        setBounds(x + offset, y + offset, width, height);
    }
}
